///////////////////////////////////////////////////////////////////////////////////
// File     : sys_handler.h
// Date     : 01/04/2012
// Author   : alain greiner and joel porquet
// Copyright (c) UPMC-LIP6
///////////////////////////////////////////////////////////////////////////////////
// The sys_handler.c and sys_handler.h files are part of the GIET-VM nano-kernel.
// It define the syscall_vector[] (at the end of this file), as well as the 
// associated syscall handlers.
///////////////////////////////////////////////////////////////////////////////////

#ifndef _SYS_HANDLER_H
#define _SYS_HANDLER_H

#include <mapping_info.h>

///////////////////////////////////////////////////////////////////////////////////
//     Syscall Vector Table (indexed by syscall index)
///////////////////////////////////////////////////////////////////////////////////

extern const void * _syscall_vector[64];

///////////////////////////////////////////////////////////////////////////////////
// This structure can be used by the vci_chbuf_dma component to transfer a stream 
// of images from two buffers in user space to the frame buffer in kernel space.
// It contains two chbuf descriptors
// - The SRC chbuf contains two buffers (buf0 & buf1), that can be in user space.
// - The DST cbuf contains one single buffer (fbf), that is the frame buffer.
///////////////////////////////////////////////////////////////////////////////////

typedef struct fbf_chbuf_s
{
    unsigned long long  buf0;     // physical address + status for user buffer 0 
    unsigned long long  buf1;     // physical address + status for user buffer 1
    unsigned long long  fbf;      // physical address + status for user buffer 0 
    unsigned int        length;   // buffer length (bytes)
    unsigned int        padding;  // for 32 bytes alignment
} fbf_chbuf_t;   

//////////////////////////////////////////////////////////////////////////////////
//    TTY related syscall handlers
//////////////////////////////////////////////////////////////////////////////////

int _sys_tty_alloc();

int _sys_tty_write( const char*  buffer,
                    unsigned int length,
                    unsigned int channel );

int _sys_tty_read(  char*        buffer,
                    unsigned int length,
                    unsigned int channel );

int _sys_tty_get_lock( unsigned int   channel,
                       unsigned int * save_sr_ptr );

int _sys_tty_release_lock( unsigned int   channel,
                           unsigned int * save_sr_ptr );

//////////////////////////////////////////////////////////////////////////////
//    TIM related syscall handlers
//////////////////////////////////////////////////////////////////////////////

int _sys_tim_alloc();

int _sys_tim_start( unsigned int period );

int _sys_tim_stop();

//////////////////////////////////////////////////////////////////////////////
//    NIC related syscall handlers
//////////////////////////////////////////////////////////////////////////////

int _sys_nic_alloc();

int _sys_nic_sync_send( void* vbuf );

int _sys_nic_sync_receive( void* vbuf );

//////////////////////////////////////////////////////////////////////////////
//    FBF related syscall handlers
//////////////////////////////////////////////////////////////////////////////

int _sys_fbf_sync_write( unsigned int offset,
                         void*        buffer,
                         unsigned int length );

int _sys_fbf_sync_read(  unsigned int offset,
                         void*        buffer,
                         unsigned int length );

int _sys_fbf_cma_alloc();

int _sys_fbf_cma_start( void*        vbase0, 
                        void*        vbase1,  
                        unsigned int length );

int _sys_fbf_cma_display( unsigned int buffer_index );

int _sys_fbf_cma_stop();

//////////////////////////////////////////////////////////////////////////////
//    Miscelaneous syscall handlers
//////////////////////////////////////////////////////////////////////////////

int _sys_ukn();

int _sys_proc_xyp( unsigned int* x,
                   unsigned int* y,
                   unsigned int* p );

int _sys_task_exit( char* string );

int _context_switch();

int _sys_local_task_id();

int _sys_global_task_id();

int _sys_thread_id();

int _sys_procs_number( unsigned int  x,
                       unsigned int  y, 
                       unsigned int* number );

int _sys_vobj_get_vbase( char*         vspace_name,
                         char*         vobj_name,
                         unsigned int* vbase );

int _sys_vobj_get_length( char*         vspace_name, 
                          char*         vobj_name,
                          unsigned int* length );

int _sys_xy_from_ptr( void*          ptr,
                      unsigned int*  x,
                      unsigned int*  y );

int _sys_heap_info( unsigned int* vaddr, 
                    unsigned int* length,
                    unsigned int  x,
                    unsigned int  y ); 

#endif

// Local Variables:
// tab-width: 4
// c-basic-offset: 4
// c-file-offsets:((innamespace . 0)(inline-open . 0))
// indent-tabs-mode: nil
// End:
// vim: filetype=c:expandtab:shiftwidth=4:tabstop=4:softtabstop=4

